"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


This module contains the core OMAG configuration class and its methods. These methods are used
to create and update OMAG server configuration documents. Configuration documents must be subsequently
deployed to take effect.

"""

import json

# import json
from pyegeria._client import Client
from pyegeria._globals import enable_ssl_check
from pyegeria._validators import (
    validate_name,
    validate_guid,
    validate_url
)


class CoreServerConfig(Client):
    """
    CoreServerConfig is a class that extends the Client class. It provides methods to configure and interact with access
    services in the OMAG server.

    Attributes:

        server_name: str
            The name of the OMAG server to configure.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None

     """

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str,
            user_pwd: str = None,
            verify_flag: bool = enable_ssl_check,
    ):
        self.admin_command_root: str
        Client.__init__(self, server_name, platform_url, user_id, user_pwd, verify_flag)
        self.admin_command_root = (
                self.platform_url
                + "/open-metadata/admin-services/users/"
                + user_id
        )

    #
    #       Configure Access Services
    #
    def get_stored_configuration(self, server_name: str = None) -> dict:
        """ Retrieves all the configuration documents for a server
        Parameters
        ----------
            self :
                the implicit instance of the class
            server_name : str, optional
               The name of the server to get the configured access services for.
               If not provided, the server name associated with the instance is used.
        Returns
        -------
        dict
            The stored configurations for the given server as a JSON dict.

        Raises
        ------
         InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values.
         PropertyServerException
             Raised by the server when an issue arises in processing a valid request.
         NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action.
        Notes
        -----

        If a server name doesn't currently exist, the method will return a starter configuration body with some of the
        key elements filled in that can serve as a starter for a valid server configuration using the fine-grained
        methods in the full OMAG Server Configuration API.

        More details on the OMAG Server Configuration at: https://egeria-project.org/guides/admin/servers/
        """

        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/configuration"
        response = self.make_request("GET", url)
        return response.json().get("omagserverConfig", "No configuration found")

    def is_server_configured(self, server_name: str = None) -> bool:
        """ Check if the server has a stored configuration

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, it uses the default server_name from the instance.

        Returns
        -------
        bool
            Returns True if the server has a stored configuration, False otherwise.
        """
        if server_name is None:
            server_name = self.server_name

        response = self.get_stored_configuration(server_name=server_name)

        if 'auditTrail' in response:
            return True
        else:
            return False

    def get_configured_access_services(self, server_name: str = None) -> list | str:
        """ Return the list of access services that are configured for this server.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to get the configured access services for.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        list | str
            A JSON list of configured access services for the specified server. If either no services or the server
            isn't found, then an explanatory string is returned.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/access-services"
        response = self.make_request("GET", url)
        return response.json().get("services", "No access services found")

    def configure_all_access_services(self, server_name: str = None) -> None:
        """ Enable all access services that are registered with this server platform.
            The access services will send notifications if it is part of its implementation.
        Parameters
        ----------
        server_name : str, optional
            The name of the server to  configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/access-services"
        self.make_request("POST", url)

    def configure_all_access_services_no_topics(self, server_name: str = None) -> None:
        """ Configure all access services for the specified server with no cohort/Event Bus.
        Parameters
        ----------
        server_name : str, optional
            The name of the server to configure.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.
        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/access-services/no-topics"
        self.make_request("POST", url)

    def clear_all_access_services(self, server_name: str = None) -> None:
        """ Disable the access services. This removes all configuration for the access services and disables the
            enterprise repository services.
        Parameters
        ----------
        server_name : str, optional
            The name of the server to clear all access services for.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        None
            This method does not return any value.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/access-services"
        self.make_request("DELETE", url)

    def get_access_service_config(self, access_service_name: str, server_name: str = None) -> dict:
        """ Retrieve the config for an access service.

        Parameters
        ----------
        access_service_name : str
            The name of the access service.

        server_name : str, optional
            The name of the server to get the configured access services for.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        dict
            The access service configuration as a JSON dictionary.

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.
       """
        if server_name is None:
            server_name = self.server_name
        validate_name(access_service_name)

        url = f"{self.admin_command_root}/servers/{server_name}/access-services/{access_service_name}"
        response = self.make_request("GET", url)
        return response.json().get("config", "Access service not found")

    def configure_access_service(self, access_service_name: str, access_service_options: dict = None,
                                 server_name: str = None) -> None:
        """ Enable a single access service. This access service will send notifications if it is part
            of its implementation.

        Parameters
        ----------
        access_service_name : str
            The name of the access service to be configured.

        access_service_options : dict, optional
            Optional configuration for the access service. For, instance, this can be used to
            set the zones that an access service has access to.

        server_name : str, optional
            The name of the server to configure access services for.
            If not provided, the server name associated with the instance is used.
        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.

        Notes
        -----
        This method configures the given access service for the specified server. If the server name is not provided,
        the default server name will be used.
        """
        if server_name is None:
            server_name = self.server_name
        validate_name(access_service_name)
        url = f"{self.admin_command_root}/servers/{server_name}/access-services/{access_service_name}"
        self.make_request("POST", url, access_service_options)

    def configure_access_service_no_topics(self, access_service_name: str, access_service_options: dict = None,
                                           server_name: str = None) -> None:
        """ Enable a single access service. Notifications, if supported, are disabled.

        Parameters
        ----------
        access_service_name : str
            The name of the access service to be configured.

        access_service_options : dict, optional
            Optional configuration for the access service. For, instance, this can be used to
            set the zones that an access service has access to.

        server_name : str, optional
            The name of the server to configure access services for.
            If not provided, the server name associated with the instance is used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.

        Notes
        -----
        This method configures the given access service for the specified server. If the server name is not provided,
        the default server name will be used. Notifications, if supported, are disabled.
        """
        if server_name is None:
            server_name = self.server_name
        validate_name(access_service_name)

        url = f"{self.admin_command_root}/servers/{server_name}/access-services/{access_service_name}/no-topics"
        self.make_request("POST", url, access_service_options)

    def clear_access_service(self, access_service_name: str, server_name: str = None) -> None:
        """ Remove the config for an access service.

        Parameters
        ----------
        access_service_name : str
            The name of the access service to be cleared.

        server_name : str, optional
            The name of the server where the access service is located. If not provided, the server name from the class
            instance will be used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
          If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
          Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
          The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
          Raised when configuration parameters passed on earlier calls turn out to be
          invalid or make the new call invalid.
        """

        if server_name is None:
            server_name = self.server_name
        validate_name(access_service_name)

        url = f"{self.admin_command_root}/servers/{server_name}/access-services/{access_service_name}"
        self.make_request("DELETE", url)

    def get_access_services_configuration(self, server_name: str = None) -> list:
        """ Return the detailed configuration for the access services in this server.

        Parameters
        ----------
        server_name : str, optional
            The name of the server to retrieve access service configurations from.
            If not provided, the server name from the class instance will be used.

        Returns
        -------
        list
            The access services configuration for the specified server, returned as a JSON list.

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/access-services/configuration"
        response = self.make_request("GET", url)

        return response.json().get("services", "No access services configured")

    #
    #       Configure Event Bus
    #
    def get_event_bus(self, server_name: str = None) -> dict:
        """ Returns the event bus configuration for the specified server

        Parameters
        ----------
        server_name: str, optional
            The name of the server to retrieve the event bus configuration from. If not provided, the default
            server name specified in the class instance will be used.

        Returns
        -------
            The event bus configuration as a JSON dictionary

        Raises
        ------
        InvalidParameterException
            If the response status code is not 200 and the related HTTP code is also not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/event-bus"

        response = self.make_request("GET", url)
        return response.json().get("config", "No event bus configured")

    def set_event_bus(self, event_bus_config: dict, server_name: str = None) -> None:
        """ Sets the event bus configuration for the server.

        Parameters
        ----------

        event_bus_config : dict
            A JSON dictionary containing the Event Bus configuration

        server_name : str, optional
            The name of the server to retrieve the event bus configuration from. If not provided, the default server
            name specified in the class instance will be used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action

        Description
        -----------
        Set up the default event bus for embedding in event-driven connector.
        The resulting connector will be used for example, in the OMRS Topic Connector for each cohort,
        the in and out topics for each Access Service and possibly the local repository's event mapper.
        When the event bus is configured, it is used only on future configuration.
        It does not affect existing configuration.

        """

        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/event-bus"
        self.make_request("POST", url, event_bus_config)

    def clear_event_bus(self, server_name: str = None) -> None:
        """ Delete the event bus configuration for the given server.

        Parameters
        ----------

        server_name : str, optional
            The name of the server to retrieve the event bus configuration from. If not provided, the default server
            name specified in the class instance will be used.

        Returns
        -------
        None

        Raises
        ------

        InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action

        Description
        -----------
        Delete the current configuration for the event bus. This does not impact that existing configuration for the
        server, only future configuration requests.
        """

        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/event-bus"
        self.make_request("DELETE", url)

    #
    #       Configure Audit Logs
    #
    def get_audit_log_destinations(self, server_name: str = None) -> dict:
        """ Get the destinations for a servers audit log

         Parameters
         ----------
         server_name : str, optional
         The name of the server to retrieve the audit configuration for. If no value is provided will pull
         the default from the class definition.

         Returns
         -------
         Returns json string containing the audit log destinations configuration.

         Raises
         ------

         InvalidParameterException
             If the client passes incorrect parameters on the request - such as bad URLs or invalid values
         PropertyServerException
             Raised by the server when an issue arises in processing a valid request
         NotAuthorizedException
             The principle specified by the user_id does not have authorization for the requested action
        """

        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations"
        response = self.make_request("GET", url)
        return response.json().get("connections", "No audit log destinations configured")

    def clear_audit_log_destinations(self, server_name: str = None) -> None:
        """ Clears the audit log destination configuration for the specified server

        Parameters
        ----------
        server_name : str
            Name of the server to clear audit log destination

        Returns
        -------

        Raises
        ------
        InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
            Raised when configuration parameters passed on earlier calls turn out to be
            invalid or make the new call invalid.

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations"
        self.make_request("DELETE", url)

    def clear_a_log_destination(self, dest_name: str, server_name: str = None) -> None:
        """  Clears audit log destinations for a server

        Parameters
        ----------

        dest_name : str
            Qualified name of the log destination to clear.
        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException:
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(dest_name)
        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/connection/{dest_name}"
        self.make_request("DELETE", url)

    def add_console_log_destinations(self, severities: [str], server_name: str = None) -> None:
        """ Adds a console log destination to a server

        Parameters
        ----------
        severities : [str]
            List of severities to send to this log destination.
        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException
            Raised when configuration parameters passed on earlier calls turn out to be
            invalid or make the new call invalid.

        """
        if server_name is None:
            server_name = self.server_name
        if severities is None:
            severities = []
        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/console"
        self.make_request("POST", url, severities)

    def add_default_log_destinations(self, server_name: str = None) -> None:
        """ Adds the default log destination to a server

        Parameters
        ----------

        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/default"
        self.make_request("POST", url)

    def add_event_topic_log_destinations(self, topic_name: str, severities: [str], server_name: str = None) -> None:
        """ Adds an event topic log destination to a server

        Parameters
        ----------

        topic_name : str
            Name of the event topic to send audit logs to.
        severities : [str], optional
            List of severities to send to the Topic

        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException:
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException:
            Raised when configuration parameters passed on earlier calls turn out to be
            invalid or make the new call invalid.

        """
        if server_name is None:
            server_name = self.server_name

        validate_name(topic_name)

        if severities is None:
            severities = []

        url = (f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/event-topic?topicName="
               f"{topic_name}")
        self.make_request("POST", url, severities)

    def add_file_log_destinations(self, directory_name: str, severities=None,
                                  server_name: str = None) -> None:
        """ Adds a file log destination to a server. Each message is a separate file in the directory
            indicated by the directory name.

        Parameters
        ----------
        directory_name : str
            Path to the audit files.
        severities : [str], optional
            List of severities to send to this destination.
        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException:
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        ConfigurationErrorException:
            Raised when configuration parameters passed on earlier calls turn out to be
            invalid or make the new call invalid.

        """
        if severities is None:
            severities = ["Error", "Exception"]
        if server_name is None:
            server_name = self.server_name

        validate_name(directory_name)

        if severities is None:
            severities = []
        url = (f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/files?directoryName="
               f"{directory_name}")
        self.make_request("POST", url, severities)

    def add_slf4j_log_destination(self, severities: [str] = None, server_name: str = None) -> None:
        """ Adds an SLF4J log destination to a server

        Parameters
        ----------
        severities: str
            List of severities to send to the destination.
        server_name : str
            Name of the server to update.

        Returns
        -------
        Void

        Raises
        ------

        InvalidParameterException:
            If the client passes incorrect parameters on the request - such as bad URLs or invalid values
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name
        if severities is None:
            severities = []
        url = f"{self.admin_command_root}/servers/{server_name}/audit-log-destinations/slf4j"
        self.make_request("POST", url, severities)

    #
    #   Basic Repository Configuration
    #
    def set_no_repository_mode(self, server_name: str = None) -> None:
        """ Disable the local repository for this server

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the method uses the value stored in `self.server_name`.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository"
        self.make_request("DELETE", url)

    def get_local_repository_config(self, server_name: str = None) -> dict:
        """ Retrieve the local repository configuration as a JSON dictionary

        Parameters
        ----------
        server_name : str, optional
            The name of the server for which to retrieve the local repository configuration. If not specified,
            the default server name will be used.

        Returns
        -------
        dict
            The local repository configuration for the specified server as a JSON dictionary.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = self.admin_command_root + "/servers/" + server_name + "/local-repository/configuration"
        response = self.make_request("GET", url)

        # return response.json().get("config")

        return response.json().get("config", "No configuration found")

    def clear_local_repository_config(self, server_name: str = None) -> None:
        """ Clear the configuration of the local repository

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not specified, the default server name will be used.

        Returns
        -------
        None
            This method does not return anything.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = self.admin_command_root + "/servers/" + server_name + "/local-repository/configuration"
        self.make_request("DELETE", url)

    def set_local_metadata_collection_id(self, metadata_collection_id: str, server_name: str = None) -> None:
        """ set the metadata collection id of the local repository

        Parameters
        ----------
        metadata_collection_id : str
            The unique identifier of the metadata collection to set.
        server_name : str, optional
            The name of the server. If not provided, defaults to the server name associated with the instance.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        validate_guid(metadata_collection_id)

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/metadata-collection-id"
        self.make_request("POST", url, metadata_collection_id)

    def get_local_metadata_collection_id(self, server_name: str = None) -> str:
        """ get the local metadata collection id
        Parameters
        ----------
                server_name : str, optional
            The name of the server. If not provided, defaults to the server name associated with the instance.

        Returns
        -------
        Metadata collection id

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        The server repository must have already been activated.
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/metadata-collection-id"
        response = self.make_request("GET", url, )
        return response.json().get('guid', "No ID found")

    def get_local_metadata_collection_name(self, server_name: str = None) -> str:
        """ get the local metadata collection name
        Parameters
        ----------
                server_name : str, optional
            The name of the server. If not provided, defaults to the server name associated with the instance.

        Returns
        -------
        Metadata collection id

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        The server repository must have already been activated.
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/metadata-collection-name"
        response = self.make_request("GET", url, )
        return response.json().get("resultString", "No name found")

    def set_local_metadata_collection_name(self, metadata_collection_name: str, server_name: str = None) -> None:
        """ Set local metadata collection name

        Parameters
        ----------
        metadata_collection_name : str
            The name of the metadata collection to be set.

        server_name : str, optional
            The name of the server where the metadata collection should be set.
            If not provided, the server name defaults to the instance's server name.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(metadata_collection_name)

        url = (f"{self.admin_command_root}/servers/{server_name}/local-repository/"
               f"metadata-collection-name/{metadata_collection_name}")
        self.make_request("POST", url)

    def set_in_mem_local_repository(self, server_name: str = None) -> None:
        """  Sets the local repository to use the native in-memory repository

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the method uses the default server name from the object.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/in-memory-repository"
        self.make_request("POST", url)

    def set_graph_local_repository(self, server_name: str = None) -> None:
        """ Sets the local repository to use JanusGraph file based repository

        Parameters
        ----------
        server_name : str, optional
            The name of the server to set as the graph local repository.
            If not provided, the current server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/local-graph-repository"
        self.make_request("POST", url)

    def set_read_only_local_repository(self, server_name: str = None) -> None:
        """ Sets the local repository to be read-only

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the default server name of the instance will be used.

        Returns
        -------
        None
            This method does not return anything.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/read-only-repository"
        self.make_request("POST", url)

    def set_repository_proxy_details(self, connector_provider: str, server_name: str = None) -> None:
        """ Sets the local repository to use the proxy repository specified by the connection.

        Parameters
        ----------
        connector_provider: str
            Specifies the class of the proxy connector provider.
        server_name : str, optional
            The name of the server. If not provided, the default server name of the instance will be used.

        Returns
        -------
        None
            This method does not return anything.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        validate_name(connector_provider)

        url = (f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/repository-proxy/"
               f"details?connectorProvider={connector_provider}")
        self.make_request("POST", url)

    def set_plug_in_repository(self, config_body: dict, server_name: str = None) -> None:
        """ Configure the metadata repository using a full repository connection body.

        Parameters
        ----------
        config_body : dict
            The configuration body for the plug-in repository. This should contain the necessary parameters for
            connecting to the repository.

        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        None
            This method does not return anything.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/plugin-repository/connection"
        self.make_request("POST", url, config_body)

    def set_xtdb_in_mem_repository(self, server_name: str = None) -> None:
        """ Set xtdb local repository connection to be XTDB with an in memory repository

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the method will use the default server name.

        Returns
        -------
        None
            This method does not return any value.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/xtdb-in-memory-repository"
        self.make_request("POST", url)

    def set_xtdb_local_kv_repository(self, server_name: str = None) -> None:
        """ Set xtdb local repository connection to be XTDB with a local file based key-value store

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the method will use the default server name.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action


        Description
        -----------
        This method is used to set the local key-value repository mode to XTDB local KV repository for a given server.
         The server name can be optionally specified, otherwise, the default server

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/xtdb-local-kv-repository"
        self.make_request("POST", url)

    def set_xtdb_local_repository(self, xtdb_config_body: dict, server_name: str = None) -> None:
        """ Set the local repository connection to be XTDB with a potentially complex XTDB configuration

        Parameters
        ----------
        xtdb_config_body : dict
            The XTDB configuration body as a json dict.

        server_name : str, optional
            The name of the server. If not provided, it uses the default server name specified in the instance.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        See https://egeria-project.org/connectors/repository/xtdb for more information

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/local-repository/mode/xtdb-local-repository"
        self.make_request("POST", url, xtdb_config_body)

    def set_xtdb_pg_repository(self, host: str, pg_user: str, pg_pwd: str, server_name: str = None) -> None:
        """ Set the local repository connection to be XTDB using PostgresSQL Server, passing in basic parameters

        Parameters
        ----------
        host : str
            the full hostname and port of the postgres server

        pg_user : str
            postgresql user name

        pg_pwd : str
            postgresql user password

        server_name : str, optional
            The name of the server. If not provided, it uses the default server name specified in the instance.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        Current convention is to use the server name as the database name. Currently, the database is assumed
        to already exist at the specified server.

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(pg_pwd)
        validate_name(pg_user)
        validate_name(host)
        pg_db = server_name.lower()

        jdbc_url = f'"jdbc:postgresql://{host}/{pg_db}?user={pg_user}&password={pg_pwd}"'
        index_dir = f'"data/servers/{server_name}/repository/xtdb/rdb-index"'
        lucene_dir = f'"data/servers/{server_name}/repository/xtdb/lucene"'

        index_str = '{:xtdb/index-store {:kv-store {:xtdb/module xtdb.rocksdb/->kv-store :db-dir '
        index_str2 = index_str + index_dir + '}}'

        lucene_str = ':xtdb.lucene/lucene-store {:db-dir ' + lucene_dir
        lucene_str2 = lucene_str + ' :indexer {:xtdb/module xtdb.lucene.egeria/->egeria-indexer} '
        lucene_str3 = lucene_str2 + ':analyzer {:xtdb/module xtdb.lucene.egeria/->ci-analyzer}} '

        conn_pool_str = ':xtdb.jdbc/connection-pool {:dialect {:xtdb/module xtdb.jdbc.psql/->dialect} '
        conn_pool_str2 = f'{conn_pool_str} :db-spec {{:jdbcUrl {jdbc_url} }} }}'

        tx_str = ':xtdb/tx-log {:xtdb/module xtdb.jdbc/->tx-log :connection-pool '
        tx_str2 = tx_str + ':xtdb.jdbc/connection-pool :poll-sleep-duration "PT1S"}'

        doc_str = ':xtdb/document-store {:xtdb/module xtdb.jdbc/->document-store :connection-pool'
        doc_str2 = doc_str + ' :xtdb.jdbc/connection-pool}}'

        edn = f"{index_str2} {lucene_str3} {conn_pool_str2} {tx_str2} {doc_str2}"
        body = {
            "xtdbConfigEDN": edn,
        }

        print(json.dumps(body, indent=4))
        url = f'{self.admin_command_root}/servers/{server_name}/local-repository/mode/xtdb-local-repository'
        self.make_request("POST", url, body)

    def get_open_metadata_archives(self, server_name: str = None) -> dict:
        """ Return the list of open metadata archives configured to load on startup.

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        dict
            A JSON dictionary of the open metadata archives configured to load on startup.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/open-metadata-archives"
        response = self.make_request("GET", url)

        return response.json().get("connections", "No archives found")

    def clear_open_metadata_archives(self, server_name: str = None) -> None:
        """ Clear open metadata archives from being loaded at startup

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, uses the default server name assigned to the instance.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/open-metadata-archives"
        self.make_request("DELETE", url)

    def add_startup_open_metadata_archive_file(self, archive_file: str, server_name: str = None) -> None:
        """ Add a metadata archive file to be loaded on startup
        Parameters
        ----------

        archive_file : str
            The file path of the metadata archive file.

        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(archive_file)

        url = f"{self.admin_command_root}/servers/{server_name}/open-metadata-archives/file"
        self.make_request("POST", url, archive_file)

    #
    #   Basic settings and security
    #
    def set_basic_server_properties(self, local_server_description: str, organization_name: str,
                                    local_server_url: str, local_server_user_id: str, local_server_password: str,
                                    max_page_size: int = 0, server_name: str = None):
        """ Sets the basic server properties.
        Parameters
        ----------
        local_server_description : str
            Description of the defined server.
        organization_name : str
            Name of the organization.
        local_server_url : str
            URL of the defined server.
        local_server_user_id : str
            User ID of the defined server.
        local_server_password : str
            Password of the defined server.
        max_page_size : int, optional
            Maximum page size for server response (default is 0).
        server_name : str, optional
            Name of the server being defined (default is None).

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        validate_name(local_server_user_id)
        validate_url(local_server_url)
        validate_name(local_server_password)

        basic_props = {
            "localServerDescription": local_server_description,
            "organizationName": organization_name,
            "localServerURL": local_server_url,
            "localServerUserId": local_server_user_id,
            "localServerPassword": local_server_password,
            "maxPageSize": max_page_size
        }
        url = self.admin_command_root + "/servers/" + server_name + "/server-properties"

        self.make_request("POST", url, basic_props)

    def get_basic_server_properties(self, server_name: str = None) -> dict | str:
        """ Retrieve the basic properties associated with this server

        Parameters
        ----------
        server_name : str, optional
            The name of the server to retrieve the local metadata collection ID from. If not provided, the current
            server name associated with the instance of the class will be used.

        Returns
        -------
        dict
            A JSON structure of the basic server properties.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = self.admin_command_root + "/servers/" + server_name + "/server-properties"
        response = self.make_request("GET", url)

        return response.json().get("basicServerProperties", "No server properties found")

    def get_server_type_classification(self, server_name: str = None) -> dict | str:
        """ Clears the server type for the given server

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If None, the default server name will be used.

        Returns
        -------
        str:
            The server type for the given server.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/server-type-classification"
        response = self.make_request("GET", url)

        return response.json().get("serverTypeClassification", "No server type found")

    def get_server_security_connection(self, server_name: str = None) -> dict | str:
        """ Retrieve the security connection configuration for a server.

        Parameters
        ----------
        server_name : str, optional
            The name of the server for which the security connection information is requested.
            If not provided, the method will use the default server name.

        Returns
        -------
        dict
            A dictionary with the connection information for the server's security.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/security/connection"
        response = self.make_request("GET", url)
        return response.json()

    def set_server_security_connection(self, security_connection_body: dict, server_name: str = None) -> None:
        """ Set the server security configuration

        Parameters
        ----------
        security_connection_body : str
            The JSON body containing the configuration details for setting up the server security connection.

        server_name : str (optional)
            The name of the server for which the security connection should be set.
            If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/security/connection"
        self.make_request("POST", url, security_connection_body)

    def clear_server_security_connection(self, server_name: str = None) -> None:
        """ Clears the server security configuration

                Parameters
                ----------

                server_name : str (optional)
                    The name of the server for which the security connection should be set.
                    If not provided, the default server name will be used.

                Returns
                -------
                None

                Raises
                ------
                InvalidParameterException
                    If the response code is not 200.
                PropertyServerException:
                    Raised by the server when an issue arises in processing a valid request
                NotAuthorizedException:
                    The principle specified by the user_id does not have authorization for the requested action

                """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/security/connection"
        self.make_request("DELETE", url)

    def get_server_classification(self, server_name: str = None) -> dict:
        """ Get server classification

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the value from `self.server_name` will be used.

        Returns
        -------
        str
            The server type classification.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/server-type-classification"
        response = self.make_request("GET", url)

        return response.json().get("serverTypeClassification")

    #
    # View Services
    #

    def get_configured_view_svcs(self, server_name: str = None) -> dict:
        """ Get the list of view services configured for the specified server

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        dict
            A JSON list containing the configured view services for the specified server.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/view-services"
        response = self.make_request("GET", url)

        return response.json().get("services", "No view services found")

    def config_all_view_services(self, mdr_server_name: str,
                                   mdr_server_platform_root_url: str, server_name: str = None) -> None:
        """ Enable all view services that are registered with this OMAG server platform.

        Parameters
        ----------

        mdr_server_name : str
            the name of the paired OMAG server.

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        mdr_server_platform_root_url : str
            The platform URL root where the metadata access server is running.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        To configure a view service specifying the complete configuration body you can use the
        full_omag_config module.
        """

        if server_name is None:
            server_name = self.server_name

        validate_name(mdr_server_name)
        validate_name(mdr_server_platform_root_url)

        view_service_body = {
            "class": "ViewServiceRequestBody",
            "omagserverName": mdr_server_name,
            "omagserverPlatformRootURL": mdr_server_platform_root_url
        }

        url = f"{self.admin_command_root}/servers/{server_name}/view-services"
        self.make_request("POST", url, view_service_body)

    def config_all_view_services_w_body(self, view_services_request_body, server_name: str = None) -> None:
        """ Configure all view services for the specified view server with a simple configuration.

        Parameters
        ----------
        view_services_request_body: dict
            a valid JSON viewServicesRequestBody object

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----


        """

        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/view-services"
        self.make_request("POST", url, view_services_request_body)

    def clear_all_view_services(self, server_name: str = None) -> None:
        """ Clears all the view services for the given server

        Parameters
        ----------
        server_name : str, optional
            The name of the server. If None, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name
        url = f"{self.admin_command_root}/servers/{server_name}/view-services"
        self.make_request("DELETE", url)

    def get_view_svc_config(self, service_url_marker: str, server_name: str = None) -> dict | str:
        """ Retrieves the view service configuration for the given view server.
        Parameters
        ----------
        service_url_marker : str
            The service URL marker. A list can be retrieved through the `list_registered_view_svcs` of the
            `registered_info` module.

        server_name : str, optional
            The server name. If not provided, the default server name will be used.

        Returns
        -------
        dict | str
            The configuration of the view service if found, or the message "No view services found" if not found.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        Details on view services configuration can be found at:
         https://egeria-project.org/guides/admin/servers/configuring-the-view-services/?h=view#integration-view-services

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(service_url_marker)

        url = f"{self.admin_command_root}/servers/{server_name}/view-services/{service_url_marker}"
        response = self.make_request("GET", url)
        return response.json().get("config", "No view services found")

    def config_view_service(self, service_url_marker: str, mdr_server_name: str,
                            mdr_server_platform_root_url: str, server_name: str = None) -> None:
        """ Configure a view service specified by the service_url_marker with basic properties.

        Parameters
        ----------
        service_url_marker : str
            The service URL marker. A list can be retrieved through the `list_registered_view_svcs` of the
            `registered_info` module.

        mdr_server_name : str
            the name of the paired OMAG server.

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        mdr_server_platform_root_url : str
            The platform URL root where the metadata access server is running.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        To configure a view service specifying the complete configuration body you can use the
        extended_omag_config module.
        """

        if server_name is None:
            server_name = self.server_name
        validate_name(service_url_marker)
        validate_name(mdr_server_name)
        validate_name(mdr_server_platform_root_url)

        view_service_body = {
            "class": "ViewServiceRequestBody",
            "omagserverName": mdr_server_name,
            "omagserverPlatformRootURL": mdr_server_platform_root_url
        }

        url = f"{self.admin_command_root}/servers/{server_name}/view-services/{service_url_marker}"
        self.make_request("POST", url, view_service_body)

    def clear_view_service(self, service_url_marker: str, server_name: str = None) -> None:
        """ Remove the view service specified by the service_url_marker.

        Parameters
        ----------
        service_url_marker : str
            The service URL marker. A list can be retrieved through the `list_registered_view_svcs` of the
            `registered_info` module.

        server_name : str, optional
            The name of the server where the service resides. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        Details on view services configuration can be found at:
         https://egeria-project.org/guides/admin/servers/configuring-the-view-services/?h=view#integration-view-services

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(service_url_marker)

        url = f"{self.admin_command_root}/servers/{server_name}/view-services/{service_url_marker}"
        self.make_request("DELETE", url)

    def get_view_svcs_config(self, server_name: str = None) -> str | list:
        """ Retrieves the view services configuration for the specified view server.
        Parameters
        ----------
        server_name : str, optional
            The name of the server for which the view services configuration is retrieved.
            If not provided, the default server name will be used.

        Returns
        -------
        str
            The JSON representation of the view services configuration.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        Details on view services configuration can be found at:
         https://egeria-project.org/guides/admin/servers/configuring-the-view-services/?h=view#integration-view-services

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/view-services/configuration"
        response = self.make_request("GET", url)

        return response.json().get("services", "No services found")

    #
    #   Cohort Configuration, etc.
    #
    def add_cohort_registration(self, cohort_name: str, server_name: str = None) -> None:
        """ Enable registration of server to an open metadata repository cohort using the default topic structure
            (DEDICATED_TOPICS). A cohort is a group of open metadata repositories that are sharing metadata.
            An OMAG server can connect to zero, one or more cohorts. Each cohort needs a unique name.
            The members of the cohort use a shared topic to exchange registration information and events
            related to the changes in their supported metadata types and instances. They are also able to query
            each other's metadata directly through REST calls.

        Parameters
        ----------
        cohort_name : str
            Name of the cohort to be registered.

        server_name : str, optional
            Name of the server to which the cohort should be added. Defaults to None.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200 after making the POST request.

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(cohort_name)

        url = f"{self.admin_command_root}/servers/{server_name}/cohorts/{cohort_name}"
        self.make_request("POST", url)

    def get_cohort_config(self, cohort_name: str, server_name: str = None) -> dict:
        """ Get the cohort configuration for the given cohort.

        Parameters
        ----------
        cohort_name : str
            Name of the cohort to get the configuration for.
        server_name : str, optional
            The name of the server. If None, the default server name will be used.

        Returns
        -------
        Dictionary containing the JSON structure of the cohort configuration.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        validate_name(cohort_name)

        url = self.admin_command_root + "/servers/" + server_name + "/cohorts/" + cohort_name
        response = self.make_request("GET", url)

        return response.json().get("config", "No cohort configuration found")

    def deploy_server_config(self, target_platform_body: dict, server_name: str = None) -> None:
        """ Add a metadata archive file to be loaded on startup
        Parameters
        ----------

        target_platform_body : dict
            The target platform URL in a JSON dict defining where to deploy the configuration to.

        server_name : str, optional
            The name of the server. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/configuration/deploy"
        self.make_request("POST", url, target_platform_body)

    #
    #   Integration Groups
    #
    def clear_all_integration_groups(self, server_name: str = None) -> None:
        """ Remove all the integration groups associated with the server.

        Parameters
        ----------

        server_name : str, optional
            The name of the server where the service resides. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----


        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/integration-groups"
        self.make_request("DELETE", url)

    def clear_an_integration_group(self, group_qualified_name: str, server_name: str = None) -> None:
        """ Remove the integration group specified by the group_qualified_name parameter.

        Parameters
        ----------
        group_qualified_name : str
            The name of the integration group to remove.

        server_name : str, optional
            The name of the server where the service resides. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        """
        if server_name is None:
            server_name = self.server_name
        validate_name(group_qualified_name)

        url = f"{self.admin_command_root}/servers/{server_name}/view-services/{group_qualified_name}"
        self.make_request("DELETE", url)

    def get_integration_groups_config(self, server_name: str = None) -> list | str:
        """ Get the Integration Groups configuration server specified by the server_name parameter.

        Parameters
        ----------

        server_name : str, optional
            The name of the server. If None, the default server name will be used.

        Returns
        -------
        Dictionary containing the JSON structure of the Integration Groups configuration.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = self.admin_command_root + "/servers/" + server_name + "/integration-groups/configuration"
        response = self.make_request("GET", url)

        return response.json().get("groups", "No Integration Group configuration found")

    def config_integration_group(self, omag_server_name: str, omag_server_platform_root_url: str,
                                 qualified_name: str, server_name: str = None) -> None:
        """ Add configuration for a single integration group to the server's config document.

        Parameters
        ----------

        omag_server_name : str
            the name of the paired OMAG server.

        qualified_name : str
            The unique name of teh integration group.

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        omag_server_platform_root_url : str
            The platform URL root where the metadata access server is running.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        See https://egeria-project.org/concepts/integration-group/ for more information.

        """

        if server_name is None:
            server_name = self.server_name
        validate_name(omag_server_name)
        validate_url(omag_server_platform_root_url)
        validate_name(qualified_name)

        integration_group_service_body = {
            "class": "IntegrationGroupConfig",
            "omagserverName": omag_server_name,
            "omagserverPlatformRootURL": omag_server_platform_root_url,
            "integrationGroupQualifiedName": qualified_name
        }

        url = f"{self.admin_command_root}/servers/{server_name}/integration-groups/configuration"
        self.make_request("POST", url, integration_group_service_body)

    #
    #   Engine Host Config
    #

    def clear_engine_definitions_client_config(self, server_name: str = None) -> None:
        """ Remove the configuration for the Governance Engine OMAS Engine client configuration in a single call.
        This overrides the current values.

        Parameters
        ----------

        server_name : str, optional
            The name of the server where the service resides. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        see also https://egeria-project.org/services/omes/
        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/engine-definitions/client-config"
        self.make_request("DELETE", url)

    def set_engine_definitions_client_config(self, mdr_server_name: str, mdr_server_platform_root_url: str,
                                             server_name: str = None) -> None:
        """ Set up the name and platform URL root for the metadata server running the Governance Engine OMAS that
         provides the governance engine definitions used by the engine services.

        Parameters
        ----------

        mdr_server_name : str
            the name of the paired OMAG server.

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        mdr_server_platform_root_url : str
            The platform URL root where the metadata access server is running.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        See https://egeria-project.org/services/omes/ for more information.

        """

        if server_name is None:
            server_name = self.server_name
        validate_name(mdr_server_name)
        validate_url(mdr_server_platform_root_url)

        body = {
            "class": "OMAGServerClientConfig",
            "omagserverName": mdr_server_name,
            "omagserverPlatformRootURL": mdr_server_platform_root_url,
        }

        url = f"{self.admin_command_root}/servers/{server_name}/engine-definitions/client-config"
        self.make_request("POST", url, body)

    def clear_engine_list(self, server_name: str = None) -> None:
        """ Remove the configuration for the Governance Engine OMAS Engine client configuration in a single call.
        This overrides the current values.

        Parameters
        ----------

        server_name : str, optional
            The name of the server where the service resides. If not provided, the default server name will be used.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----

        """
        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/engine-list"
        self.make_request("DELETE", url)

    def set_engine_list(self, engine_list: [dict], server_name: str = None) -> None:
        """ Set up the list of governance engine that will use the metadata from the same metadata access server
         as the engine host uses for retrieving the engine configuration.

        Parameters
        ----------

        engine_list: [dict]
            a JSON dict containing a list of tuples for the engine - see below for structure.

        server_name : str, optional
            The name of the metadata access server that is to provide the metadata services to the view service.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        Notes
        -----
        See https://egeria-project.org/concepts/governance-engine/ for more information.

        engine_list structure is:

        [
            {
                "class" : "EngineConfig",
                # engineId is optional - one is automatically created if needed
                "engineId": "string",
                "engineQualifiedName": "string",
                "engineUserId": "string"
            }
        ]

        """

        if server_name is None:
            server_name = self.server_name

        url = f"{self.admin_command_root}/servers/{server_name}/engine-list"
        self.make_request("POST", url, engine_list)

    def get_engine_host_services_config(self, server_name: str = None) -> dict | str:
        """ Return the configuration for the complete engine host services in this server.

        Parameters
        ----------

        server_name : str, optional
            The name of the server. If None, the default server name will be used.

        Returns
        -------
        List containing the JSON structure of the Integration Groups configuration.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        if server_name is None:
            server_name = self.server_name

        url = self.admin_command_root + "/servers/" + server_name + "/engine-host-services/configuration"
        response = self.make_request("GET", url)

        # return response.json().get("config", "No engine definitions client configuration found")
        return response.json().get("services", "No engine host services")

    def get_placeholder_variables(self) -> dict:
        """ get placeholder variables

        Get the placeholder variables from the platform.

        Returns:
            dict: A dictionary containing the placeholder variables.

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action



        """
        url = self.admin_command_root + "/stores/placeholder-variables"
        response = self.make_request("GET", url)

        return response.json().get("stringMap")

    def set_placeholder_variables(self, placeholder_variables: dict) -> None:
        """ Set placeholder variables - replaces previous placeholders with the new list

        Parameters
        ----------
        placeholder_variables : dict
            A dictionary containing the placeholder variables.

        Returns
        -------
        None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        url = f"{self.admin_command_root}/stores/placeholder-variables"
        self.make_request("POST", url, placeholder_variables)

    def clear_placeholder_variables(self) -> None:
        """
        Clears the placeholder variables for a store.

        Parameters:
        - self: The instance of the class.

        Returns:
        - None

        Raises
        ------
        InvalidParameterException
            If the response code is not 200.
        PropertyServerException:
            Raised by the server when an issue arises in processing a valid request
        NotAuthorizedException:
            The principle specified by the user_id does not have authorization for the requested action

        """
        url = f"{self.admin_command_root}/stores/placeholder-variables"
        self.make_request("DELETE", url)



#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Display the status of cataloged platforms and servers.
"""

import argparse
import os, sys
import time

from rich.console import Console
from rich.live import Live
from rich.table import Table

from pyegeria import RuntimeManager
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))

disable_ssl_warnings = True
console = Console(width=200)


def display_status(server: str, url: str, username: str, user_pass: str, jupyter: bool = EGERIA_JUPYTER,
                   width: int = EGERIA_WIDTH):
    r_client = RuntimeManager(server, url, username)
    token = r_client.create_egeria_bearer_token(username, user_pass)

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(
            title=f"Server Status for Platform - {time.asctime()}",
            # style="bold white on black",
            row_styles=["bold white on black"],
            header_style="white on dark_blue",
            title_style="bold white on black",
            caption_style="white on black",
            caption=f"Status of Platforms - '{url}'",
            show_lines=True,
            # expand=True
        )
        table.add_column("Platform Name")
        # table.add_column("Platform GUID")
        table.add_column("Platform URL")
        table.add_column("Platform Origin")
        table.add_column("Description")
        table.add_column("Platform Started")
        table.add_column("Servers")

        server_types = {
            "Metadata Access Store": "Store",
            "View Server": "View",
            "Engine Host Server": "EngineHost",
            "Integration Daemon": "Integration"
        }
        try:
            platform_list = r_client.get_platforms_by_type()
            if type(platform_list) is str:
                print("No OMAG Server Platforms found?")
                sys.exit(1)

            for platform in platform_list:
                platform_name = platform['properties'].get("displayName", '---')
                platform_guid = platform['elementHeader']["guid"]
                platform_desc = platform['properties'].get("resourceDescription",'---')
                server_list = ""

                platform_report = r_client.get_platform_report(platform_guid)
                platform_url = platform_report.get('platformURLRoot', " ")
                platform_origin = platform_report.get("platformOrigin", " ")
                platform_started = platform_report.get("platformStartTime", " ")

                servers = platform_report.get("omagservers", None)

                if servers is not None:
                    for server in servers:
                        server_name = server.get("serverName", " ")
                        server_type = server.get("serverType", " ")
                        server_status = server.get("serverActiveStatus", "UNKNOWN")
                        if server_status in ("RUNNING", "STARTING"):
                            status_flag = "[bright green]"
                        elif server_status in ("INACTIVE", "STOPPING"):
                            status_flag = "[bright red]"
                        else:
                            server_status = "UNKNOWN"
                            status_flag = "[bright yellow]"

                        serv = f"{status_flag}{server_types[server_type]}: {server_name}\n"
                        server_list = server_list + serv

                    table.add_row(platform_name, platform_url, platform_origin, platform_desc,
                                  platform_started, server_list, style="bold white on black")

        except (Exception) as e:
        # console.print_exception(e)
            platform_url = " "
            platform_origin = " "
            platform_started = " "

        return table

    try:
        with Live(generate_table(), refresh_per_second=4, screen=True) as live:
            while True:
                time.sleep(2)
                live.update(generate_table())

    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)

    except KeyboardInterrupt:
        pass
    finally:
        r_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_VIEW_SERVER
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    display_status(server, url, userid, user_pass)


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This module allows users to query the available (registered) capabilities of Egeria. Detailed information is returned
to provide both insight and understanding in how to use these capabilities. For example, when configuring an Egeria
integration service, it is importregistered_info.pyant to know what companion service it depends on so that you can make sure the
companion service is also configured and running.

"""

from pyegeria import (
    Client, max_paging_size, InvalidParameterException, PropertyServerException, UserNotAuthorizedException,
    print_exception_response
)



class LoadedResources(Client):
    """ Client to search and retrieve currently loaded information about connectors, templates, governance actions,
        etc.

    Attributes:
    ----------
        server_name: str
                Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None
        verify_flag: bool
            Flag to indicate if SSL Certificates should be verified in the HTTP requests.
            Defaults to False.

    Methods:
    -------

    """

    admin_command_root: str

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str,
            user_pwd: str = None,
            verify_flag: bool = False,
    ):
        if server_name is None:
            server_name = "NA"
        Client.__init__(self, server_name, platform_url,
                        user_id, user_pwd, verify_flag)

    def get_all_templates(self, server: str = None, start_from: int = 0, page_size: int = max_paging_size) -> list | str:
        """ Get Loaded templates for the Server.

           Parameters
           ----------

           Returns
           -------
           dict | str
              A dictionary containing a simplified list of key template attributes.

           Raises
           ------
           InvalidParameterException
               If the response code is not 200.
           PropertyServerException:
               Raised by the server when an issue arises in processing a valid request
           NotAuthorizedException:
               The principle specified by the user_id does not have authorization for the requested action

           """
        server = self.server_name if server is None else server
        url = (f"{self.platform_url}/servers/{server}/open-metadata/framework-services/asset-owner/open-metadata-store/"
               f"users/{self.user_id}/metadata-elements/by-search-string?startFrom=start_from&pageSize=page_size")
        body = {
                  "class" : "SearchStringRequestBody",
                  "searchString" : ".*Template.*"
                }
        response = self.make_request("POST", url, body)
        return response.json().get("elementList", "No elements")

    # def get_all_assets_in_archives(self, asset_type: str = None, archive_id: str = None, server: str= None, page_start:int = 0, page_size = max_paging_size):




"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.



This script restarts an integration daemon.

"""

import os
from rich import print, print_json
from rich.console import Console

import time
import click
# from ops_config import Config, pass_config
from pyegeria import ServerOps, AutomatedCuration, INTEGRATION_GUIDS, Platform
from pyegeria._exceptions import (
    InvalidParameterException,
    PropertyServerException,
    UserNotAuthorizedException,
    print_exception_response,
)




@click.command('stop')
@click.pass_context
def stop_server(ctx):
    """Stop an engine-host daemon"""
    try:
        c = ctx.obj
        p_client = Platform(c.engine_host, c.engine_host_url, c.admin_user, c.admin_user_password)

        p_client.shutdown_server()

        click.echo(f"Stopped server {c.engine_host}")
    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)


@click.command('start')
@click.pass_context
def start_server(ctx):
    """Start or restart an engine-host from its known configuration """
    try:
        c = ctx.obj
        p_client = Platform(c.engine_host, c.engine_host_url, c.admin_user, c.admin_user_password)

        p_client.activate_server_stored_config()

        click.echo(f"Started server {c.engine_host}")

    except (InvalidParameterException, PropertyServerException) as e:
        print_exception_response(e)




"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Runtime manager is a view service that supports user interaction with the running platforms.

"""
import asyncio
from requests import Response

from pyegeria._exceptions import (
    InvalidParameterException,
)
from pyegeria import Client, AutomatedCuration, max_paging_size, body_slimmer



class RuntimeManager(Client):
    """
    Client to issue Runtime status requests.

    Attributes:

        server_name: str
                Name of the server to use.
        platform_url : str
            URL of the server platform to connect to
        user_id : str
            The identity of the user calling the method - this sets a default optionally used by the methods
            when the user doesn't pass the user_id on a method call.
        user_pwd: str
            The password associated with the user_id. Defaults to None
        verify_flag: bool
            Flag to indicate if SSL Certificates should be verified in the HTTP requests.
            Defaults to False.

    Methods:

    """

    def __init__(
            self,
            server_name: str,
            platform_url: str,
            user_id: str,
            user_pwd: str = None,
            verify_flag: bool = False,
        ):
        Client.__init__(self, server_name, platform_url, user_id,
                          user_pwd, verify_flag)
        self.cur_command_root = f"{platform_url}/servers/"
        self.platform_guid = "44bf319f-1e41-4da1-b771-2753b92b631a" # this is platform @ 9443 from the core content archive
        self.default_platform_name = "Default Local OMAG Server Platform" # this from the core content archive

    async def _async_get_platforms_by_name(self, filter: str = None, server:str = None, effective_time: str = None,
                                           start_from: int = 0, page_size: int = max_paging_size )-> str | list:
        """ Returns the list of platforms with a particular name. The name is specified in the filter.  Async version.
            Parameters
            ----------
           filter : str, opt
                Filter specifies the display name or qualified name of the platforms to return information for. If the
                value is None, we will default to the default_platform_name that comes from the core content pack.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.

            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        if filter is None:
            filter = self.default_platform_name


        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/platforms/by-name?"
               f"startFrom={start_from}&pageSize={page_size}")
        if effective_time is not None:
            body = {
                "filter" : filter,
                "effectiveTime" : effective_time
            }
        else:
            body = {
                "filter": filter
            }

        response = await self._async_make_request("POST", url, body)

        return response.json().get('elementList','No platforms found')


    def get_platforms_by_name(self, filter: str = None, server: str = None, effective_time: str = None,
                              start_from: int = 0, page_size: int = max_paging_size) -> str | list:
        """ Returns the list of platforms with a particular name. The name is specified in the filter.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the display name or qualified name of the platforms to return information for. If the
                value is None, we will default to the default_platform_name that comes from the core content pack.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException
        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_platforms_by_name(filter, server,
                                                                             effective_time,
                                                                            start_from, page_size))
        return response

    async def _async_get_platforms_by_type(self, filter: str = None, server:str = None, effective_time: str = None,
                                           start_from: int = 0, page_size: int = max_paging_size )-> str | list:
        """ Returns the list of platforms with a particular deployed implementation type.  The value is specified in
            the filter. If it is null, or no request body is supplied, all platforms are returned.  Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
                   The name of the server to get governance engine summaries from. If not provided, the default server name
                   will be used.

            start_from : int, optional
                   The index from which to start fetching the engine actions. Default is 0.

            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        if filter is None:
            filter = "OMAG Server Platform"

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/platforms/"
               f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}")

        if effective_time is not None:
            body = {
                "filter": filter,
                "effectiveTime": effective_time
            }
        else:
            body = {
                "filter": filter
            }

        response = await self._async_make_request("POST", url, body)
        return response.json().get('elements','No platforms found')

    def get_platforms_by_type(self, filter: str = None, server:str = None, effective_time: str = None,
                              start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of platforms with a particular deployed implementation type.  The value is specified in
            the filter. If it is null, or no request body is supplied, all platforms are returned.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_platforms_by_type(filter, server, effective_time,
                                                                             start_from, page_size))
        return response

    async def _async_get_platform_templates_by_type(self, filter: str = None, server:str = None, effective_time: str = None,
                                           start_from: int = 0, page_size: int = max_paging_size )-> str | list:
        """ Returns the list of platform templates for a particular deployed implementation type.  The value is
            specified in the filter. If it is null, or no request body is supplied, all platforms are returned.
            Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
                   The name of the server to get governance engine summaries from. If not provided, the default server name
                   will be used.

            start_from : int, optional
                   The index from which to start fetching the engine actions. Default is 0.

            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        if filter is None:
            filter = "OMAG Server Platform"

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/platforms/"
               f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}&getTemplates=true")

        if effective_time is not None:
            body = {
                "filter": filter,
                "effectiveTime": effective_time
            }
        else:
            body = {
                "filter": filter
            }

        response = await self._async_make_request("POST", url, body)
        return response.json().get('elements','No platforms found')


    def get_platform_templates_by_type(self, filter: str = None, server:str = None, effective_time: str = None,
                              start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of platform templates with a particular deployed implementation type.  The value is
            specified in the filter. If it is null, or no request body is supplied, all platforms are returned.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_platforms_by_type(filter, server, effective_time,
                                                                             start_from, page_size))
        return response

    async def _async_get_platform_report(self, platform_guid: str = None, server:str = None) -> str | list:
        """ Returns details about the running platform. Async version.

            Parameters
            ----------
            platform_guid : str
                The unique identifier for the platform.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            Returns
            -------
            Response
               A json dict with the platform report.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/platforms/{platform_guid}/report")

        response = await self._async_make_request("GET", url)

        return response.json().get('element','No platforms found')

    def get_platform_report(self, platform_guid: str = None, server: str = None) -> str | list:
        """ Returns details about the running platform.

            Parameters
            ----------
            platform_guid : str
                The unique identifier for the platform.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.


            Returns
            -------
            Response
               A json dict with the platform report.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_platform_report(platform_guid, server))
        return response

    async def _async_get_platform_by_guid(self, platform_guid: str = None, server:str = None,
                                           effective_time: str = None) -> str | list:
        """ Returns details about the platform's catalog entry (asset). Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

        """
        if server is None:
           server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/platforms/{platform_guid}")

        if effective_time is not None:
            body = {
                "effectiveTime": effective_time
            }
            response = await self._async_make_request("POST", url, body)

        else:
            response = await self._async_make_request("POST", url)

        return response.json().get('elements','No platforms found')

    def get_platform_by_guid(self, platform_guid: str = None, server: str = None,
                                effective_time: str = None) -> str | list:
        """ Returns details about the platform's catalog entry (asset).

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_platforms_by_guid(platform_guid, server,
                                                                             effective_time))
        return response

    async def _async_get_server_by_guid(self, server_guid: str, server:str = None,
                                           effective_time: str = None) -> str | dict:
        """ Returns details about the server's catalog entry (asset). Async version.

            Parameters
            ----------
            server_guid : str
                The unique identifier for the platform.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/{server_guid}")

        if effective_time is not None:
            body = {
                "effectiveTime": effective_time
            }
            response = await self._async_make_request("POST", url, body)

        else:
            response = await self._async_make_request("POST", url)

        return response.json().get('elements','No server found')

    def get_server_by_guid(self, server_guid: str, server: str = None,
                                effective_time: str = None) -> str | dict:
        """ Returns details about the platform's catalog entry (asset).

            Parameters
            ----------
            server_guid : str
                The unique identifier for the platform.

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_server_by_guid(server_guid, server, effective_time))
        return response


    async def _async_get_servers_by_name(self, filter: str, server:str = None, effective_time: str = None,
                                        start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of servers with a particular name.  The name is specified in the filter. Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/by-name?"
               f"startFrom={start_from}&pageSize={page_size}")

        if effective_time is None:
            body = {
                "filter": filter
            }
        else:
            body = {
                "filter": filter,
                "effective_time": effective_time
            }
        response = await self._async_make_request("POST", url, body)

        return response.json().get('elements','No platforms found')

    def get_servers_by_name(self, filter: str, server: str = None) -> str | list:
        """ Returns the list of servers with a particular name.  The name is specified in the filter.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_servers_by_name(filter, server))
        return response

    async def _async_get_servers_by_dep_impl_type(self, filter: str = "*", server:str = None, effective_time: str = None,
                                        start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of servers with a particular deployed implementation type. The value is specified
            in the filter. If it is null, or no request body is supplied, all servers are returned.
            Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        if filter == '*':
            filter = None

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/"
               f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}")

        body = body_slimmer({ "filter": filter, "effective_time": effective_time})

        response = await self._async_make_request("POST", url, body)

        return response.json().get('elements','No platforms found')

    def get_servers_by_dep_impl_type(self, filter: str = "*", server:str = None, effective_time: str = None,
                                     start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of servers with a particular deployed implementation type.
            The value is specified in the filter. If it is null, or no request body is supplied,
            all servers are returned.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_servers_by_dep_impl_type(filter, server, effective_time,
                                                                                    start_from, page_size))
        return response

    async def _async_get_server_templates_by_dep_impl_type(self, filter: str = "*", server:str = None, effective_time: str = None,
                                        start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of server templates with a particular deployed implementation type.   The value is
            specified in the filter. If it is null, or no request body is supplied, all servers are returned.
            Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        if filter == '*':
            filter = None

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/"
               f"by-deployed-implementation-type?startFrom={start_from}&pageSize={page_size}&getTemplates=true")

        body = body_slimmer({ "filter": filter, "effective_time": effective_time})

        response = await self._async_make_request("POST", url, body)

        return response.json().get('elements','No platforms found')

    def get_server_templates_by_dep_impl_type(self, filter: str = "*", server:str = None, effective_time: str = None,
                                     start_from: int = 0, page_size: int = max_paging_size ) -> str | list:
        """ Returns the list of server templates with a particular deployed implementation type.
            The value is specified in the filter. If it is null, or no request body is supplied,
            all servers are returned.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_server_templates_by_dep_impl_type(filter, server, effective_time,
                                                                                    start_from, page_size))
        return response


    async def _async_get_server_by_guid(self, server_guid: str = None, server:str = None,
                                           effective_time: str = None) -> str | list:
        """ Returns details about the server's catalog entry (asset). Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
           server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/{server_guid}")

        if effective_time is not None:
            body = {
                "effectiveTime": effective_time
            }
            response = await self._async_make_request("POST", url, body)

        else:
            response = await self._async_make_request("POST", url)

        return response.json().get('element','No servers found')

    def get_server_by_guid(self, server_guid: str = None, server: str = None,
                                effective_time: str = None) -> str | list:
        """ Returns details about the server's catalog entry (asset). Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            start_from : int, optional
               The index from which to start fetching the engine actions. Default is 0.


            page_size : int, optional
               The maximum number of engine actions to fetch in a single request. Default is `max_paging_size`.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_server_by_guid(server_guid, server,
                                                                             effective_time))
        return response


    async def _async_get_server_report(self, server_guid: str = None, server: str = None) -> str | list:
        """ Returns details about the running server. Async version.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.


            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        if server is None:
            server = self.server_name

        url = (f"{self.platform_url}/servers/{server}/api/open-metadata/runtime-manager/software-servers/{server_guid}/report")

        response = await self._async_make_request("GET", url)

        return response.json().get('elements', 'No server found')


    def get_server_report(self, server_guid: str = None, server: str = None) -> str | list:
        """ Returns details about the running server.

            Parameters
            ----------
            filter : str, opt
                Filter specifies the kind of deployed implementation type of the platforms to return information for.
                If the value is None, we will default to the "OMAG Server Platform".

            server : str, optional
               The name of the server to get governance engine summaries from. If not provided, the default server name
               will be used.

            Returns
            -------
            Response
               A lit of json dict with the platform reports.

            Raises
            ------
            InvalidParameterException
            PropertyServerException
            UserNotAuthorizedException

        """
        loop = asyncio.get_event_loop()
        response = loop.run_until_complete(self._async_get_server_report(server_guid, server))
        return response




#!/usr/bin/env python3
"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Unit tests for the Utils helper functions using the Pytest framework.


A simple server status display
"""
import argparse
import os
import time

from rich.live import Live
from rich.table import Table

from pyegeria._exceptions import (InvalidParameterException, PropertyServerException, UserNotAuthorizedException,
                                  print_exception_response, )
from pyegeria.core_omag_server_config import CoreServerConfig
from pyegeria.server_operations import ServerOps

disable_ssl_warnings = True

EGERIA_METADATA_STORE = os.environ.get("EGERIA_METADATA_STORE", "active-metadata-store")
EGERIA_KAFKA_ENDPOINT = os.environ.get('KAFKA_ENDPOINT', 'localhost:9092')
EGERIA_PLATFORM_URL = os.environ.get('EGERIA_PLATFORM_URL', 'https://localhost:9443')
EGERIA_VIEW_SERVER = os.environ.get('VIEW_SERVER', 'view-server')
EGERIA_VIEW_SERVER_URL = os.environ.get('EGERIA_VIEW_SERVER_URL', 'https://localhost:9443')
EGERIA_INTEGRATION_DAEMON = os.environ.get('INTEGRATION_DAEMON', 'integration-daemon')
EGERIA_INTEGRATION_DAEMON_URL = os.environ.get('EGERIA_INTEGRATION_DAEMON_URL', 'https://localhost:9443')
EGERIA_ADMIN_USER = os.environ.get('ADMIN_USER', 'garygeeke')
EGERIA_ADMIN_PASSWORD = os.environ.get('ADMIN_PASSWORD', 'secret')
EGERIA_USER = os.environ.get('EGERIA_USER', 'erinoverview')
EGERIA_USER_PASSWORD = os.environ.get('EGERIA_USER_PASSWORD', 'secret')
EGERIA_JUPYTER = bool(os.environ.get('EGERIA_JUPYTER', 'False'))
EGERIA_WIDTH = int(os.environ.get('EGERIA_WIDTH', '200'))


def display_status(server: str, url: str, username: str, user_pass: str, jupyter: bool = EGERIA_JUPYTER,
                   width: int = EGERIA_WIDTH):
    p_client = ServerOps(server, url, username)
    c_client = CoreServerConfig(server, url, username, user_pass)

    def generate_table() -> Table:
        """Make a new table."""
        table = Table(title=f"Server Status for Platform - {time.asctime()}", style="bold white on black",
                      row_styles=["bold white on black"], header_style="white on dark_blue",
                      title_style="bold white on black",
                      caption_style="white on black", caption=f"Server Status for Platform - '{url}'", show_lines=True,
                      # expand=True
                      )

        table.add_column("Known Server")
        table.add_column("Status")
        table.add_column("Server Type")
        table.add_column("Server Description")
        known_server_list = p_client.get_known_servers()
        active_server_list = p_client.get_active_server_list()
        if len(known_server_list) == 0:
            return table

        for server in known_server_list:
            if server in active_server_list:
                status = "Active"
            else:
                status = "Inactive"
            server_type = c_client.get_server_type_classification(server)["serverTypeName"]
            description = c_client.get_basic_server_properties(server).get("localServerDescription", " ")

            table.add_row(server, "[red]Inactive" if status == "Inactive" else "[green]Active", server_type,
                          description)

        return table

    try:
        with Live(generate_table(), refresh_per_second=4, screen=True) as live:
            while True:
                time.sleep(2)
                live.update(generate_table())

    except (InvalidParameterException, PropertyServerException, UserNotAuthorizedException) as e:
        print_exception_response(e)
    except KeyboardInterrupt:
        pass

    finally:
        p_client.close_session()
        c_client.close_session()


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--server", help="Name of the server to display status for")
    parser.add_argument("--url", help="URL Platform to connect to")
    parser.add_argument("--userid", help="User Id")
    parser.add_argument("--password", help="User Password")
    args = parser.parse_args()

    server = args.server if args.server is not None else EGERIA_METADATA_STORE
    url = args.url if args.url is not None else EGERIA_PLATFORM_URL
    userid = args.userid if args.userid is not None else EGERIA_ADMIN_USER
    user_pass = args.password if args.password is not None else EGERIA_USER_PASSWORD

    display_status(server, url, userid, user_pass)


if __name__ == "__main__":
    main()



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.

Definitions, utilities and exceptions in support of the Egeria Python Client package.

"""

import json
from enum import Enum

"""

The following definitions are used in creating Exception messages. 
They mirror similar definitions in the Egeria core. 
Note that not all of the definitions are currently used - they merely serve as placeholders for future extensions.

"""


class EgeriaErrorCode(Enum):
    """ Egeria error codes """

    def __str__(self):
        return (
                "http_error_code="
                + self.value["http_error_code"]
                + "messageId="
                + self.value["message_id"]
                + ", message="
                + self.value["message_template"]
                + ", systemAction="
                + self.value["system_action"]
                + ", userAction="
                + self.value["user_action"]
        )


class OMAGServerInstanceErrorCode(EgeriaErrorCode):
    """ OMAGServer instance error codes """

    BAD_SERVER_SECURITY_CONNECTION = dict(
        https_error_code="400",
        message_id="OMAG-MULTI-TENANT-400-001",
        message_template="The OMAG server {0} has been configured with a bad connection to its security connector."
                         + " Error message is {1}. Connection is {2}",
        system_action="The system is unable to validate the users issuing requests to this server.",
        user_action="Review the error message to determine the cause of the problem.",
    )

    """
        OMAG-MULTI-TENANT-400-002 - The OMAG server {0} has been requested to shut down but the following services are
        still running: {1}
    """
    SERVICES_NOT_SHUTDOWN = dict(
        https_error_code="400",
        message_id="OMAG-MULTI-TENANT-400-002",
        message_template="The OMAG server {0} has been requested to shutdown but the following services " +
                         "are still running: {1}",
        system_action="The system is unable to shutdown the server correctly.",
        user_action="Review other error messages to determine the cause of the problem."
                    + " This is likely to be a logic error in the services listed in the message",
    )

    """
        OMAG-MULTI-TENANT-400-003 - Method {0} called on behalf of the {1} service is unable to create a client-side
        open metadata topic connection because the topic name is not configured in the configuration for this service.
    """
    NO_TOPIC_INFORMATION = dict(
        https_error_code="400",
        message_id="OMAG-MULTI-TENANT-400-003",
        message_template="Method {0} called on behalf of the {1} service is unable to create a client-side open "
                         + "metadata topic connection because the topic name is not configured in the configuration "
                         + "for this service.",
        system_action="This is a configuration error and an exception is sent to the requester.",
        user_action="Correct the configuration of the access service to include the name of the topic.",
    )

    """
        OMAG-MULTI-TENANT-400-004 - The connector provider class name {0} does not create a connector of class {1} 
        which is required for the {2}
    """
    NOT_CORRECT_CONNECTOR_PROVIDER = dict(
        https_error_code="400",
        message_id="OMAG-MULTI-TENANT-400-004",
        message_template="The connector provider class name {0} does not create a connector of class {1} which is"
                         + " required for the {2}",
        system_action="An invalid parameter exception is returned to the caller.",
        user_action="Either change the connector or the hosting environment because the current"
                    + " combination is not compatible.",
    )

    """
        OMAG-MULTI-TENANT-404-001 - The OMAG Server {0} is not available to service a request from user {1}
    """
    SERVER_NOT_AVAILABLE = dict(
        https_error_code="404",
        message_id="OMAG-MULTI-TENANT-404-001",
        message_template="The OMAG Server {0} is not available to service a request from user {1}",
        system_action="The system is unable to process the request because the server"
                      + " is not running on the called platform.",
        user_action="Verify that the correct server is being called on the correct platform and that this server "
                    + "is running. Retry the request when the server is available.",
    )

    """
        OMAG-MULTI-TENANT-404-002- The {0} service is not available on OMAG Server {1} to handle a request from user {2}
    """
    SERVICE_NOT_AVAILABLE = dict(
        https_error_code="404",
        message_id="OMAG-MULTI-TENANT-404-002",
        message_template="The {0} service is not available on OMAG Server {1} to handle a request from user {2}",
        system_action="The system is unable to process the request because the service is not available.",
        user_action="Verify that the correct server is being called on the correct platform and that the "
                    + "requested service is configured to run there.  "
                    + "Once the correct environment is in place, retry the request.",
    )

    """
        OMAG-MULTI-TENANT-404-003 - The server name is not available for the {0} operation
    """
    SERVER_NAME_NOT_AVAILABLE = dict(
        https_error_code="404",
        message_id="OMAG-MULTI-TENANT-404-003",
        message_template="The server name is not available for the {0} operation",
        system_action="The system is unable to return the server name because it is not available.",
        user_action="Check that the server where the access service is running initialized correctly.  "
        + "Correct any errors discovered and retry the request when the open metadata services are available.",
    )

    """
        OMAG-MULTI-TENANT-404-004 - The open metadata repository services are not initialized for the {0} operation
    """
    OMRS_NOT_INITIALIZED = dict(
        https_error_code="404",
        message_id="OMAG-MULTI-TENANT-404-004",
        message_template="The open metadata repository services are not initialized for the {0} operation",
        system_action="The system is unable to connect to the open metadata repository services because"
                      + " they are not running in this server.",
        user_action="Check that the server where the called service is running initialized correctly.  "
        + "Correct any errors discovered and retry the request when the open metadata services are available.",
    )

    """
        OMAG-MULTI-TENANT-404-005 - The open metadata repository services are not available for the {0} operation
    """
    OMRS_NOT_AVAILABLE = (
        dict(
            https_error_code="404",
            message_id="OMAG-MULTI-TENANT-404-005",
            message_template="The open metadata repository services are not available for the {0} operation",
            system_action="The system is unable to connect to the open metadata repository services because"
                          " they are not in the correct state to be called.",
            user_action="Check that the server where the called service is running initialized correctly and is not"
                        + " in the process of shutting down. Correct any errors discovered and retry the"
                        + " request when the open metadata repository services are available.",
        ),
    )

    """
        OMAG-MULTI-TENANT-500-003 - Method {0} called on behalf of the {1} service detected a {2} exception when
        creating an open metadata topic connection because the connector provider is incorrect.  
        The error message was {3}
    """
    BAD_TOPIC_CONNECTOR_PROVIDER = dict(
        http_error_code="500",
        message_id="OMAG-MULTI-TENANT-500-003",
        message_template="Method {0} called on behalf of the {1} service detected a {2} exception when creating an "
        + "open metadata topic connection because the connector provider is incorrect.  The error message was {3}",
        system_action="This is an internal error.  The access service is not using a valid connector provider.",
        user_action="Raise an issue on Egeria's GitHub and work with the Egeria community to resolve.",
    )


class OMAGCommonErrorCode(EgeriaErrorCode):
    CLIENT_SIDE_REST_API_ERROR = dict(
        http_error_code="503",
        message_id="CLIENT-SIDE-REST-API-CONNECTOR-503-002",
        message_template="A client-side error {0} was received by method {1} from API call {2} during the call {3}."
                         + " The error message was {4}",
        system_action="The client has issued a call to the open metadata access service REST API in a remote server"
                      + " and has received an exception from the local client libraries.",
        user_action="Review the error message to determine the cause of the error. Check that the server is running"
                    + " and the URL is correct. Look for errors in the local server's console to understand and"
                    + " correct the cause of the error. Then rerun the request",
    )

    EXCEPTION_RESPONSE_FROM_API = dict(
        http_error_code="503",
        message_id="SERVER-SIDE-REST-API-ERROR-503-003 ",
        message_template="A {0} exception was received from REST API call {1} to server {2}: error message was: {3}",
        system_action="The system has issued a call to an open metadata access service REST API in a remote server"
                      + " and has received an exception response.",
        user_action="The error message should indicate the cause of the error. "
                    + "Otherwise look for errors in the remote server's audit log and console to understand and "
                    + "correct the source of the error.",
    )

    SERVER_URL_NOT_SPECIFIED = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-001",
        message_template="The OMAG Server Platform URL is null",
        system_action="The system is unable to identify the OMAG Server Platform.",
        user_action="Create a new client and pass the URL for the server on the constructor.",
    )

    SERVER_URL_MALFORMED = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-002",
        message_template="The OMAS Server URL: {0} is not in a recognized format",
        system_action="The system is unable to connect to the OMAG Server Platform to fulfill any requests.",
        user_action="Create a new client and pass the correct URL for the server on the constructor.",
    )

    SERVER_NAME_NOT_SPECIFIED = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-003",
        message_template="The OMAG Server name is null",
        system_action="The system is unable to locate to the OMAG Server to fulfill any request.",
        user_action="Create a new client and pass the correct name for the server on the constructor.",
    )

    NULL_USER_ID = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-004",
        message_template="The user identifier {0} passed on the operation is null",
        system_action="The system is unable to process the request without a user id..",
        user_action="Correct the code in the caller to provide the user id.",
    )

    NULL_GUID = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-005",
        message_template="The unique identifier (guid) passed is null or not a string",
        system_action="The system is unable to process the request without a guid.",
        user_action="Correct the code in the caller to provide the guid.",
    )

    NULL_NAME = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-006",
        message_template="The name passed on the parameter of the operation is null",
        system_action="The system is unable to process the request without a name.",
        user_action="Correct the code in the caller to provide the name on the parameter.",
    )

    NULL_ARRAY_PARAMETER = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-007",
        message_template="The array value passed on the {0} parameter of the {1} operation is null or empty",
        system_action="The system is unable to process the request without this value.",
        user_action="Correct the code in the caller to provide the array.",
    )

    NEGATIVE_START_FROM = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-008",
        message_template="The starting point for the results {0}, passed on the {1} parameter of the {2}"
                         + " operation, is negative",
        system_action="The system is unable to process the request with this invalid value."
        + "It should be zero for the start of the values, or a number greater than 0"
        + "to start partway down the list.",
        user_action="Correct the code in the caller to provide a non-negative value for the starting point.",
    )

    NEGATIVE_PAGE_SIZE = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-009",
        message_template="The page size for the results {0}, passed on the {1} parameter of the {2} operation, " +
                         "is negative",
        system_action="The system is unable to process the request with this invalid value. "
                      + "It should be zero to return all the result, or greater than zero to set a maximum.",
        user_action="Correct the code in the caller to provide a non-negative value for the page size.",
    )

    MAX_PAGE_SIZE = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-010",
        message_template=(
                "The number of records to return, {0}, passed on the {1} parameter of the {2} operation, "
                + "is greater than the allowable maximum of {3}"
        ),
        system_action="The system is unable to process the request with this page size value.",
        user_action="Correct the code in the caller to provide a smaller page size.",
    )

    NULL_ENUM = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-012",
        message_template="The enumeration value passed on the {0} parameter of the {1} operation is null",
        system_action="The system is unable to process the request without a enumeration value.",
        user_action="Correct the code in the caller to provide the enumeration value.",
    )

    NULL_TEXT = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-013",
        message_template="The text field passed on the {0} parameter of the {1} operation is null",
        system_action="The system is unable to process the request without this text value.",
        user_action="Correct the code in the caller to provide a value in the text field.",
    )

    NULL_OBJECT = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-015",
        message_template="The object passed on the {0} parameter of the {1} operation is null",
        system_action="The system is unable to process the request without this object.",
        user_action="Correct the code in the caller to provide the object.",
    )

    NULL_SEARCH_STRING = dict(
        http_error_code="400",
        message_id="OMAG-COMMON-400-022",
        message_template="The search string passed on the {0} parameter of the {1} operation is null",
        system_action="The system is unable to process the request without a search string.",
        user_action="Correct the code in the caller to provide the search string.",
    )


class EgeriaException(Exception):
    """
    Define the Egeria exceptions raised during error handling. Modeled on the exceptions defined in the Egeria core.

    """
    raw_error_message = ""

    def __init__(self, response_body) -> None:
        response_dict = json.loads(response_body)
        self.response_class = response_dict["class"]
        self.related_http_code = response_dict["relatedHTTPCode"]
        self.exception_class_name = response_dict["exceptionClassName"]
        self.action_description = response_dict["actionDescription"]
        self.exception_error_message = response_dict["exceptionErrorMessage"]
        self.exception_error_message_id = response_dict.get("exceptionErrorMessageId", "UNKNOWN-ERROR-CODE")

        # self.exception_error_message_id = response_dict["exceptionErrorMessageId"]
        self.exception_error_message_parameters = response_dict.get("exceptionErrorMessageParameters", "{}")
        self.exception_system_action = response_dict.get("exceptionSystemAction", "UNKNOWN-SYSTEM-ACTION")
        self.exception_user_action = response_dict.get("exceptionUserAction", "UNKNOWN-USER-ACTION")

    def __str__(self):
        return self.exception_error_message


class InvalidParameterException(EgeriaException):
    """Exception due to invalid parameters such as one of the parameters is null or invalid"""

    def __init__(self, response_body):
        EgeriaException.__init__(self, response_body)


class PropertyServerException(EgeriaException):
    """Exception due to a problem retrieving information from the property server"""

    def __init__(self, response_body):
        EgeriaException.__init__(self, response_body)


class UserNotAuthorizedException(EgeriaException):
    """Exception as the requesting user is not authorized to issue this request"""

    def __init__(self, response_body):
        EgeriaException.__init__(self, response_body)


def print_exception_response(e: EgeriaException):
    """ Prints the exception response """

    if isinstance(e, EgeriaException):
        print(f"\n\nException: {e.response_class}")
        print(f"\t\t   Error Message: {e.exception_error_message}")
        print(
            f"\t\t   Error Code: {e.exception_error_message_id} with http code {str(e.related_http_code)}"
        )
        # print(f"\t\t   Raw Error Text is {e.raw_error_message}")
        print(f"\t\t   Class: {e.exception_class_name}")
        print(f"\t\t   Caller: {e.action_description}")
        print(f"\t\t   System Action: {e.exception_system_action}")
        print(f"\t\t   User Action: {e.exception_user_action}")
    else:
        print(f"\n\n\t  Not an Egeria exception {e}")



"""
SPDX-License-Identifier: Apache-2.0
Copyright Contributors to the ODPi Egeria project.


"""
import pyegeria
# from .ops_config import Config

# import monitor_gov_eng_status
# import monitor_server_status
# import monitor_server_list
# import monitor_integ_daemon_status
# import monitor_platform_status
# import monitor_engine_activity
# import refresh_integration_daemon
# import restart_integration_daemon
# import integration_daemon_actions
# import list_catalog_targets
